module Main exposing (main)

import Html exposing (..)
import Html.Attributes exposing (srcdoc)
import Playground as P exposing (..)
import Tuple exposing (first, second)


type alias Memory =
    { dx : Number
    }


type alias Bounds =
    { ymin : Number
    , ymax : Number
    , xmin : Number
    , xmax : Number
    }


type alias Coords =
    ( Number, Number )


init : Memory
init =
    { dx = 0.8
    }


main = game


game =
    P.game view update init


view : Computer -> Memory -> List Shape
view computer memory =
    let
        bounds =
            { ymin = 0
            , ymax = 32 * computer.screen.height / computer.screen.width
            , xmin = 0
            , xmax = 32
            }

        at =
            placeAtCoords computer bounds

        atCorner =
            placeCornerAtCoords computer bounds

        fx x = x * x * x / 32 / 32
    in
    List.append
        (makeRectangles computer atCorner fx bounds memory [])
        <|
            List.append
                (makeCircles computer fx bounds memory [])
                ([ P.words black "Welcome to my math project! Use arrow keys to adjust the width of each rectangle."
                    |> at ( (bounds.xmax - bounds.xmin) / 2, bounds.ymax - 1)
                , P.words black ("The current dx is " ++ (String.fromFloat memory.dx))
                    |> at ( (bounds.xmax - bounds.xmin) / 2, bounds.ymax - 2)
                ])



makeCircles : Computer -> (Number -> Number) -> Bounds -> Memory -> List Shape -> List Shape
makeCircles computer fx bounds memory prev =
    let
        resolution = (bounds.xmax - bounds.xmin) / 512
        xcoord = toFloat (List.length prev) * resolution
        ycoord = fx xcoord
        newCirc =
            circle purple 2
                |> placeAtCoords computer bounds (xcoord, ycoord)
    in
    if
        xcoord < bounds.xmax
    then
        makeCircles computer fx bounds memory <| List.append prev [ newCirc ]
    else
        prev



makeRectangles : Computer -> (Coords -> Corner -> (Number, Number) -> Shape -> Shape) -> (Number -> Number) -> Bounds -> Memory -> List Shape -> List Shape
makeRectangles computer atCorner fx bounds memory prev =
    let
        xcoord = toFloat (List.length prev) * memory.dx
        ycoord = fx xcoord
        widthOfCell = computer.screen.width / (bounds.xmax - bounds.xmin)
        newRec = 
            rectangle red (memory.dx * widthOfCell) (ycoord * widthOfCell)
                |> atCorner (xcoord, ycoord) TopLeft (memory.dx, ycoord)
    in
    if
        xcoord < bounds.xmax
    then
        makeRectangles computer atCorner fx bounds memory <| List.append prev [ newRec ]

    else
        prev


update : Computer -> Memory -> Memory
update computer memory =
    let
        step = 0.02
        threshold = 0.02
    in
    { memory
        | dx = 
            (toFloat
            <| round
            <| toFloat 100 * (
                if memory.dx > threshold then
                    memory.dx + step * toX computer.keyboard
                else
                    computer.keyboard
                        |> toX
                        |> max 0
                        |> (*) step
                        |> (+) threshold
            )) / toFloat 100
    }


placeAtScreen : Computer -> Coords -> Shape -> Shape
placeAtScreen computer coords shape =
    let
        x =
            first coords

        y =
            second coords
    in
    shape |> P.move (-computer.screen.width / 2 + x) (-computer.screen.height / 2 + y)


placeAtCoords : Computer -> Bounds -> Coords -> Shape -> Shape
placeAtCoords computer bounds (x, y) shape =
    placeAtScreen
        computer 
        ( computer.screen.width * x / (bounds.xmax - bounds.xmin)
        , computer.screen.height * y / (bounds.ymax - bounds.ymin)
        )
        shape


type Corner = TopLeft | TopRight | BottomLeft | BottomRight

placeCornerAtCoords : Computer -> Bounds -> Coords -> Corner -> (Number, Number) -> Shape -> Shape
placeCornerAtCoords computer bounds (x, y) corner (width, height) shape =
    shape
        |> placeAtCoords computer bounds
            (case corner of
                TopLeft ->
                    (x + width / 2, y - height / 2)
                TopRight ->
                    (x - width / 2, y - height / 2)
                BottomLeft ->
                    (x + width / 2, y + height / 2)
                BottomRight ->
                    (x - width / 2, y + height / 2)
            )
